@@ -283,6 +283,9 @@ sub reserve_slot
 
         $self->log_( 2, "reserve_slot selected random number $r" );
 
+        # Avoid another POPFile process using the same committed id
+        $self->db__()->begin_work;
+
         # TODO Replace the hardcoded user ID 1 with the looked up
         # user ID from the session key
 
@@ -290,6 +293,7 @@ sub reserve_slot
                  "select id from history where committed = $r limit 1;");
 
         if ( defined( $test ) ) {
+            $self->db__()->commit;
             next;
         }
 
@@ -305,6 +309,7 @@ sub reserve_slot
 
     my $result = $self->db__()->selectrow_arrayref(
                  "select id from history where committed = $r limit 1;");
+    $self->db__()->commit;
 
     my $slot = $result->[0];
 
@@ -501,6 +506,7 @@ sub commit_history__
         return;
     }
 
+    $self->db__()->begin_work;
     foreach my $entry (@{$self->{commit_list__}}) {
         my ( $session, $slot, $bucket, $magnet ) = @{$entry};
 
@@ -639,6 +645,7 @@ sub commit_history__
             $self->release_slot( $slot );
         }
     }
+    $self->db__()->commit;
 
     $self->{commit_list__} = ();
     $self->force_requery__();
@@ -721,6 +728,7 @@ sub start_deleting
     my ( $self ) = @_;
 
     $self->{classifier__}->tweak_sqlite( 1, 1, $self->db__() );
+    $self->db__()->begin_work;
 }
 
 #----------------------------------------------------------------------------
@@ -735,6 +743,7 @@ sub stop_deleting
 {
     my ( $self ) = @_;
 
+    $self->db__()->commit;
     $self->{classifier__}->tweak_sqlite( 1, 0, $self->db__() );
 }
 
